/*
 * Copyright (c) 2005 Ichiro Satoh
 * All Rights Reserved.
 * written by Ichiro Satoh
 *
 * This program was developed by Ichiro Satoh at National Institute 
 * of Informatics as a part of a mobile object system, titled 
 * "AgentSpace".  All the programs of the system is developed by 
 * Ichiro Satoh and all rights are reserved by Ichiro Satoh. 
 *
 * Permission to use, copy, modify and distribute this software and
 * its documentation is hereby granted, provided that both the
 * copyright notice and this permission notice appear in all copies of
 * the software, derivative works or modified versions, and any
 * portions thereof, and that both notices appear in supporting 
 * documentation.
 */

/**
 * @author Ichiro Satoh
 * Created on 2005/05/4
 */

package agentspace;

import java.io.IOException;
import java.net.URL;
import java.util.Hashtable;

import agentspace.context.ContextManager;
import agentspace.network.SocketClient;
import agentspace.runtime.AgentContainer;
import agentspace.runtime.AgentRuntimeManager;

public class AgentManager {
	static ContextManager manager = null;
	static private AgentManager _instance = null;
	static Hashtable containerTable = null;

	static public AgentManager instance() {
		if(_instance == null) {
			_instance = new AgentManager();
		}
		return _instance;
	}

	public AgentManager() {
		if (manager == null) {
			manager = new ContextManager(new AgentRuntimeManager());
			agentspace.monitor.AgentView view = new agentspace.monitor.AgentView(manager);
			manager.addAgentRuntimeListener(view);
		}
		if (containerTable == null){ 
			containerTable = new Hashtable();
		}
	}
	
	public AgentIdentifier receive(byte[] data) {
		AgentContainer ac = manager.load(data, true);
		return ac.getIdentifier();
	}


	public byte[] store(AgentIdentifier aid) 
	throws IOException, IllegalAgentStateException, NoSuchAgentException {
		AgentContainer ac =(AgentContainer)containerTable.get(aid);
		if (ac != null) {
			return manager.store(ac);
		}
		return null;
	}

	public void send(AgentIdentifier aid, URL url) 
	throws IOException, IllegalAgentStateException {
//		System.out.println("AgentManager: send("+aid+") > "+url);
		AgentContainer ac =(AgentContainer)containerTable.get(aid);
		if (ac != null) {
			byte[] data = manager.store(ac);
//        	System.out.println("AgentManager: data = "+data.length);
        	SocketClient.moveContent(url, data);
		}
	}
}
