/*
 * Copyright (c) 2005 Ichiro Satoh
 * All Rights Reserved.
 * written by Ichiro Satoh
 *
 * This program was developed by Ichiro Satoh at National Institute 
 * of Informatics as a part of a mobile object system, titled 
 * "AgentSpace".  All the programs of the system is developed by 
 * Ichiro Satoh and all rights are reserved by Ichiro Satoh. 
 *
 * Permission to use, copy, modify and distribute this software and
 * its documentation is hereby granted, provided that both the
 * copyright notice and this permission notice appear in all copies of
 * the software, derivative works or modified versions, and any
 * portions thereof, and that both notices appear in supporting 
 * documentation.
 */

/**
 * @author Ichiro Satoh
 * Created on 2005/05/4
 */


package agentspace;

import java.io.IOException;
import java.io.Serializable;
import java.lang.reflect.InvocationTargetException;
import java.net.URL;

import agentspace.context.AgentContextManager;

public class Context implements Serializable {
	transient private AgentContextManager manager;
	private Object target;

	public Context(AgentContextManager manager) {
		this.manager = manager;
	}
	
	public AgentIdentifier getIdentifier() {
		return manager.getIdentifier();
	}

	public String getName() {
		return manager.getName();
	}

	public void setName(String name) {
		manager.setName(name);
	}
	
	public void terminate() {
		manager.terminate();
	}
	
	public void move(URL url) throws IOException, InvalidURLException {
		manager.move(url);
	}
		
	public void move(AgentIdentifier aid, URL url) 
	throws IOException, InvalidURLException {
		manager.move(aid, url);
	}
		
	public void store(String filename) {
		System.out.println("Context store("+filename+")");
		manager.store(filename);
	}

	public void duplicate() throws IllegalAgentStateException {
		manager.duplicate();
	}
	
	public AgentIdentifier[] getAgents() {
		return manager.getAgents();
	}
	
	public AgentIdentifier[] getAgents(String name) {
		return manager.getAgents(name);
	}
	
	public Object invoke(AgentIdentifier aid, Message msg) 
	throws NoSuchAgentException, NoSuchMethodException, 
	IllegalAgentStateException, IllegalAccessException {
		try {
			return manager.invoke(aid, msg);
		}
		catch (Exception ex) {
			if (ex instanceof IllegalAgentStateException) {
				throw new IllegalAgentStateException(ex.getMessage()+" with "+aid);
			}
			if (ex instanceof NoSuchMethodException) {
				throw new NoSuchMethodException(msg+" in "+aid);
			}
			if (ex instanceof InvocationTargetException) {
				throw new IllegalAgentStateException(ex.getMessage()+" with "+aid);
			}
			if (ex instanceof IllegalAccessException) {
				throw new IllegalAccessException(ex.getMessage());
			}
			if (ex instanceof IllegalArgumentException) {
				throw new NoSuchMethodException(ex.getMessage()+" with "+aid);
			}
			else {
				throw new IllegalAccessException(ex.getMessage());
			}
		}
	}
}
