/*
 * Copyright (c) 2005 Ichiro Satoh
 * All Rights Reserved.
 * written by Ichiro Satoh
 *
 * This program was developed by Ichiro Satoh at National Institute 
 * of Informatics as a part of a mobile object system, titled 
 * "AgentSpace".  All the programs of the system is developed by 
 * Ichiro Satoh and all rights are reserved by Ichiro Satoh. 
 *
 * Permission to use, copy, modify and distribute this software and
 * its documentation is hereby granted, provided that both the
 * copyright notice and this permission notice appear in all copies of
 * the software, derivative works or modified versions, and any
 * portions thereof, and that both notices appear in supporting 
 * documentation.
 */

/**
 * @author Ichiro Satoh
 * Created on 2005/05/8
 */

package agentspace.context;

import java.net.URL;
import java.util.Hashtable;

import agentspace.AgentIdentifier;
import agentspace.Future;
import agentspace.Message;
import agentspace.NoSuchAgentException;
import agentspace.NoSuchResourceException;
import agentspace.registry.NotBoundException;
import agentspace.runtime.AgentContainer;
import agentspace.runtime.AgentRuntimeManager;

public class AgentContextManager extends ContextManager {
	AgentContainer container;
	
	public AgentContextManager(AgentRuntimeManager manager, AgentContainer container) {
		super(manager);
		this.container = container;
	}
	
	public AgentIdentifier getIdentifier() {
		return container.getIdentifier();
	}

	public void setName(String name) {
		container.setName(name);
	}
	
	public String getName() {
		return container.getName();
	}
	
	public byte[] getLocalContent(String name) throws NoSuchResourceException {
		byte[] data = null;
		Hashtable table = container.getResources();
		if (table != null) {
			data = (byte[])table.get(name);
		}
		return data;
	}

	public void move(AgentIdentifier aid, URL url) {
		try {
			move(container, url,  false);
		}
		catch (Exception ex) {
			ex.printStackTrace();
		}
	}
	
	public void store(String name) {
		class StoreHandler implements Runnable {
			String name;
			public StoreHandler(String name) {
				this.name = name;
			}
			public void run() {
				manager.store(container, name);
			}
		}
		StoreHandler handler = new StoreHandler(name);
		manager.getThreadPool().startThread(handler);
	}

	public void move(URL url) {
		try {
			move(container, url,  false);
		}
		catch (Exception ex) {
			ex.printStackTrace();
		}
	}

	public void terminate() {
		terminate(container);
	}
	
	public void duplicate() {
		class DuplicateHandler implements Runnable {
			AgentContainer ac;
			public DuplicateHandler(AgentContainer ac) {
				this.ac = ac;
			}
			public void run() {
				manager.duplicate(ac);
			}
		}
		DuplicateHandler handler = new DuplicateHandler(container);
		manager.getThreadPool().startThread(handler);
	}
	
	public AgentIdentifier[] getAgents() {
		return manager.getAgents();
	}
	
	public String getAgentName(AgentIdentifier aid) throws NoSuchAgentException { 
		AgentContainer ac = manager.getContainer(aid);
		return ac.getName();
	}
	
	public AgentIdentifier[] getAgents(String key) {
		try {
			return manager.getAgents(key);
		}
		catch (NotBoundException ex) {
			return null;
		}
	}

	public String getName(AgentIdentifier aid) throws NoSuchAgentException {
		return manager.getName(aid);
	}
	
	public Object invoke(AgentIdentifier aid, Message msg)
	throws Exception {
		if (msg.getFuture() == null) {
			msg.setFuture(new Future());
		}
		if (aid == null) {
			throw new NoSuchAgentException();
		}
		if (msg == null) {
			throw new NoSuchMethodException();
		}
		Future future = manager.invoke(aid, msg);
		return future.getReply();
	}
}