/*
 * Copyright (c) 2005 Ichiro Satoh
 * All Rights Reserved.
 * written by Ichiro Satoh
 *
 * This program was developed by Ichiro Satoh at National Institute 
 * of Informatics as a part of a mobile object system, titled 
 * "AgentSpace".  All the programs of the system is developed by 
 * Ichiro Satoh and all rights are reserved by Ichiro Satoh. 
 *
 * Permission to use, copy, modify and distribute this software and
 * its documentation is hereby granted, provided that both the
 * copyright notice and this permission notice appear in all copies of
 * the software, derivative works or modified versions, and any
 * portions thereof, and that both notices appear in supporting 
 * documentation.
 */

/**
 * @author Ichiro Satoh
 * Created on 2005/05/12
 */

package agentspace.monitor.table;

import java.awt.FileDialog;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowEvent;
import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;

import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.event.TableModelEvent;
import javax.swing.event.TableModelListener;

import agentspace.IllegalAgentStateException;
import agentspace.InvalidURLException;
import agentspace.context.ContextManager;
import agentspace.monitor.dialog.AgentDestinationDialog;
import agentspace.runtime.AgentContainer;

public class AgentTableControl implements ActionListener, TableModelListener {
	public static String DEFAULT_DESTINATION = "matp://localhost:7000"; 
	protected URL DEFAULT_URL;

	private int theCurrentRow = -1;
	protected AgentTableModel model = null;
	protected ContextManager manager = null;
	protected JFrame frame = null;
	protected AgentTableToolBar toolBar = null;
	
	public AgentTableControl(ContextManager manager, JFrame frame) {
		this.manager = manager;
		this.frame = frame;
	}

	public void setTableModel(AgentTableModel model) {
		this.model = model;
	}

	public void tableChanged(TableModelEvent e) {
		theCurrentRow = e.getColumn();
	}

	public void actionPerformed(ActionEvent evt) {
		try {
			String action = evt.getActionCommand();
			if (evt.getID() == WindowEvent.WINDOW_CLOSING) {
				System.exit(0);
			} 
			else if (action.equals("Shutdown")) {
				System.exit(0);
			} 
			else if (action.equals("Load")) {
				String filename = getFileName("*.jar");
				if (filename == null) {
					throw new IOException("No such file: "+filename); 
				}
				byte[] data = null;
				try {
					java.io.FileInputStream inStream = new java.io.FileInputStream(filename);
					java.io.ByteArrayOutputStream byteout = new java.io.ByteArrayOutputStream();
					byte[] bytes = new byte[1024];
					int count;
					while ((count = inStream.read(bytes)) != -1) {
						byteout.write(bytes, 0, count);
					}
					data = byteout.toByteArray();
				}				
				catch (Exception ex) {
					throw new IOException("Cannot read file: "+filename); 
				}
				if (data == null) {
					throw new IOException("Cannot read file: "+filename); 					
				}
				try {
					manager.load(data);
				}
				catch (Exception ex) {
					throw new Exception("Cannot load agent from file: "+filename); 
				}
			}
			else if (action.equals("Destroy")) {
				AgentContainer ac = model.getElementAt(model.getSelectedIndex());
				manager.terminate(ac);
			} 
			else if (action.equals("Halt")) {
				AgentContainer ac = model.getElementAt(model.getSelectedIndex());
				manager.halt(ac);				
			}
			else if (action.equals("Store")) {
				AgentContainer ac = model.getElementAt(model.getSelectedIndex());
				if (ac != null) {
					String filename = getFileName("*.jar");
					if (filename != null && filename.length() !=0){
						byte[] data = manager.store(ac);
						java.io.FileOutputStream fos = new java.io.FileOutputStream(filename);
						fos.write(data, 0, data.length);
						fos.close();
					}
					else {
						throw new Exception("Invalid file name.");
					}
				}
				else {
					throw new Exception("Cannot find agent");
				}
			} 
			else if (action.equals("Information")) {
				AgentContainer ac = model.getElementAt(model.getSelectedIndex());
				if (ac != null) {
					System.out.println(">> ************************************");
					System.out.println(">> "+ac);
					System.out.println(">> ************************************");
				}
			} 
			else if (action.equals("Copy")) {
				AgentContainer ac = model.getElementAt(model.getSelectedIndex());
				if (ac != null) {
					try {
						manager.duplicate(ac);
					}
					catch (Exception ex) {
						throw new Exception("Cannot duplicate the agent");											
					}
				}
				else {
					throw new Exception("Cannot find agent");					
				}
			} 
			else if (action.equals("Move")) {
				AgentContainer ac = model.getElementAt(model.getSelectedIndex());
				if (ac != null) {
					if (DEFAULT_URL == null) {
						try {
							DEFAULT_URL = new URL(DEFAULT_DESTINATION);
						} 
						catch (MalformedURLException e) {
							e.printStackTrace();
						}
					}
					AgentDestinationDialog dialog = new AgentDestinationDialog(frame, DEFAULT_URL);
					URL url = dialog.getURL();
					System.out.println("AgentTableControl: url = "+url);
					if (url != null &&  url.getHost() != null && url.getPort() > 0) {
						try {
							manager.move(ac, url, true);
							DEFAULT_URL = url;
						}
						catch (InvalidURLException ex) {
							throw new Exception("Invalid URL: "+ex.getMessage());
						}
						catch (IllegalAgentStateException ex) {
							throw new Exception("Illegal lifecycle state "+ex.getMessage());
						}
						catch (Exception ex) {
							throw new Exception("Cannot migrate agent: "+ac.getName());																		
						}
					}
					else {
						throw new Exception("Invalid host name or port number");											
					}
				}
				else {
					throw new Exception("Cannot find agent");					
				}
	        }
		} 
		catch (Exception ex) {
			JOptionPane.showMessageDialog(frame, ex.getMessage(), "Warning", JOptionPane.WARNING_MESSAGE);
//			ex.printStackTrace();
		}
	    
	}

	public String getFileName(String pattern) {
		String filename = null; 
		try {
			FileDialog fd = new FileDialog(frame, "Load Agent");
			if (pattern == null) {
				pattern = "*.*";
			}
			fd.setFile(pattern);
			fd.setDirectory(".");
			fd.setVisible(true);
			if (fd.getDirectory() != null && fd.getFile() != null) {
				return fd.getDirectory() + fd.getFile();
			} 
		}
		catch (Exception ex) {}
		return filename;
	}
}

