/*
 * Copyright (c) 2005 Ichiro Satoh
 * All Rights Reserved.
 * written by Ichiro Satoh
 *
 * This program was developed by Ichiro Satoh at National Institute 
 * of Informatics as a part of a mobile object system, titled 
 * "AgentSpace".  All the programs of the system is developed by 
 * Ichiro Satoh and all rights are reserved by Ichiro Satoh. 
 *
 * Permission to use, copy, modify and distribute this software and
 * its documentation is hereby granted, provided that both the
 * copyright notice and this permission notice appear in all copies of
 * the software, derivative works or modified versions, and any
 * portions thereof, and that both notices appear in supporting 
 * documentation.
 */

/**
 * @author Ichiro Satoh
 * Created on 2005/05/6
 */

package agentspace.network;

import java.io.IOException;
import java.net.ServerSocket;
import java.net.Socket;
import agentspace.AgentManager;

public class AgentSocketServer implements Runnable {
	final static public int SOCKET_TIMEOUT = 1000;
	protected ServerSocket listenSocket = null;
	protected ThreadGroup threadGroup = null;
	protected AgentManager manager = null;

	public AgentSocketServer(AgentManager manager, int port) throws IOException {
		this(manager, port, SOCKET_TIMEOUT);
	}

	public AgentSocketServer(AgentManager manager, int port, int timeout) throws IOException {
		this.manager = manager;
		listenSocket = new ServerSocket(port);
		threadGroup = new ThreadGroup("SERVER");
	}
	
	public void start() {
		(new Thread(threadGroup, this)).start();
	}

    public synchronized void run() {
		System.out.println("SocketServer: starting... "+listenSocket);
		while (true) {
			try {
				Socket clientSocket = listenSocket.accept();
//				System.out.println("SocketServer: clientSocket "+clientSocket);
				(new AgentServerHandlerFactory(manager, clientSocket, SOCKET_TIMEOUT)).run();
			}
			catch (IOException ex) {
				ex.printStackTrace();
			}
		}
	}

	protected synchronized void close() throws IOException {
		listenSocket.close();
		listenSocket = null;
	}

	public boolean isOpened() {
		if (listenSocket == null) {
			return false;
		} else {
			return true;
		}
	}

}