/*
 * Copyright (c) 2005 Ichiro Satoh
 * All Rights Reserved.
 * written by Ichiro Satoh
 *
 * This program was developed by Ichiro Satoh at National Institute 
 * of Informatics as a part of a mobile object system, titled 
 * "AgentSpace".  All the programs of the system is developed by 
 * Ichiro Satoh and all rights are reserved by Ichiro Satoh. 
 *
 * Permission to use, copy, modify and distribute this software and
 * its documentation is hereby granted, provided that both the
 * copyright notice and this permission notice appear in all copies of
 * the software, derivative works or modified versions, and any
 * portions thereof, and that both notices appear in supporting 
 * documentation.
 */

/**
 * @author Ichiro Satoh
 * Created on 2005/05/6
 */

package agentspace.network;

import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import java.net.InetAddress;
import java.net.Socket;
import java.net.SocketException;
import java.net.URL;
import java.net.URLConnection;
import java.net.URLStreamHandler;
import java.util.Hashtable;

public class AgentURLConnection extends URLConnection {
	final static public int DEFAULT_PORT = 7000;
	protected DataOutputStream dout = null;
	protected DataInputStream din = null;
	protected Socket sock = null;
	protected int timeout = 0;
	protected Hashtable cache = null;
	protected boolean keepalive = false;

	public AgentURLConnection(URL url) {
		super(url);
		super.setDoOutput(true);
		super.setDoInput(true);
		super.url = url;
		cache = new Hashtable();
	}

	public void setKeepAlive(boolean keepalive) {
		this.keepalive = keepalive;
	}

	public synchronized void disconnect() throws IOException {
		if (connected) {
			connected = false;
			sock.close();
			sock = null;
		}
	}
	public void connect() throws IOException {
		InetAddress dst = InetAddress.getByName(super.getURL().getHost());
		int port = super.getURL().getPort();
		if (port == -1) {
			port = DEFAULT_PORT;
		}
		try {
			sock = new Socket(dst, port);
			sock.setSoTimeout(timeout);
			sock.setTcpNoDelay(true);
			connected = true;
			dout = new DataOutputStream(sock.getOutputStream());
			din = new DataInputStream(sock.getInputStream());
		} 
		catch (SocketException e) {
			connected = false;
			sock = null;
			throw e;
		} 
		catch (IOException e) {
			sock.close();
			connected = false;
			sock = null;
			throw e;
		}
	}

	public void putContent(byte[] content) throws IOException {
		if (!connected) {
			connect();
		}
		try {
			String protocol = url.getProtocol();
			String className = "agentspace.network.protocol." + protocol.toLowerCase() + ".URLHandler";
			URLStreamHandler handler = (URLStreamHandler)cache.get(className);
			if (handler == null) {
				Object obj = Class.forName(className).newInstance();
				if (obj != null && obj instanceof URLStreamHandler) {
					handler = (URLStreamHandler) obj;
					cache.put(className, handler);
				} 
				else {
					throw new IOException();
				}
			}
			System.err.println("AgentURLConnection: url "+url+", data = "+content.length);
		} 
		catch (ClassNotFoundException e) {
			e.printStackTrace();
		} 
		catch (InstantiationException e) {
			e.printStackTrace();
		} 
		catch (IllegalAccessException e) {
			e.printStackTrace();
		} 
		catch (IOException e) {
			disconnect();
			throw e;
		}
		if (!keepalive) {
			disconnect();
		}
	}

	public boolean usingProxy() {
		return false;
	}
}