/*
 * Copyright (c) 2005 Ichiro Satoh
 * All Rights Reserved.
 * written by Ichiro Satoh
 *
 * This program was developed by Ichiro Satoh at National Institute 
 * of Informatics as a part of a mobile object system, titled 
 * "AgentSpace".  All the programs of the system is developed by 
 * Ichiro Satoh and all rights are reserved by Ichiro Satoh. 
 *
 * Permission to use, copy, modify and distribute this software and
 * its documentation is hereby granted, provided that both the
 * copyright notice and this permission notice appear in all copies of
 * the software, derivative works or modified versions, and any
 * portions thereof, and that both notices appear in supporting 
 * documentation.
 */

/**
 * @author Ichiro Satoh
 * Created on 2005/05/7
 */

package agentspace.network.protocol.matp;

import java.io.IOException;
import agentspace.AgentManager;
import agentspace.AgentIdentifier;
import agentspace.IllegalAgentStateException;
import agentspace.NoSuchAgentException;
import agentspace.network.protocol.Request;
import agentspace.network.protocol.Response;
import agentspace.network.protocol.ResponseFactory;

public class RequestHandler implements agentspace.network.RequestHandler {
	Request request;
	AgentManager manager;
	
	public RequestHandler() {
		super();
	}

	public void setRequest(Request request) {
		this.request = request;
	}

	public Response handle() {
		Response response = null;
		String method = request.getMethod();
		if (method.equals("MOVE")) {
			response = handleMOVEMethod();
		} 
		else if (method.equals("HELLO")) {
			response = handleHELLOMethod();
		} 
		else {
			response = handleUnknownMethod(method);
		}
		return response;
	}

	protected Response handleHELLOMethod() {
		ResponseFactory response = new ResponseFactory();
		response.setProtocol("MATP");
		response.setVersion("1.0");
		try {
			String name = request.getURI();
			response.setCode(MatpConstants.MATP_OK);
			response.setComment("OK");
		} 
		catch (Exception e) {
			e.printStackTrace();
			response.setCode(MatpConstants.MATP_SERVER_ERROR);
			response.setComment("Server errors");
		}
		return response;
	}

	protected Response handleMOVEMethod() {
		ResponseFactory response = new ResponseFactory();
		response.setProtocol("MATP");
		response.setVersion("1.0");
		try {
			String name = request.getURI();
			byte[] content = request.getContent();
			manager.receive(content);
			response.setCode(MatpConstants.MATP_OK);
			response.setComment("OK");
		} 
		catch (Exception e) {
			e.printStackTrace();
			response.setCode(MatpConstants.MATP_SERVER_ERROR);
			response.setComment("Server errors");
		}
		return response;
	}

	protected Response handleFETCHMethod() {
		ResponseFactory response = new ResponseFactory();
		response.setProtocol("MATP");
		response.setVersion("1.0");
		try {
			String name = request.getURI();
			AgentIdentifier aid = new AgentIdentifier(name);
			byte[] data = manager.store(aid);
			response.setContent(data);
			response.setCode(MatpConstants.MATP_OK);
		} catch (IllegalAgentStateException e) {
			response.setCode(MatpConstants.MATP_ILLEGAL_AGENT_STATE);
			response.setComment("Illegal Agent State");
		} catch (NoSuchAgentException e) {
			response.setCode(MatpConstants.MATP_NOT_FOUND);
			response.setComment("Not Found");
		} catch (SecurityException e) {
			response.setCode(MatpConstants.MATP_FORBIDDEN);
			response.setComment("Forbidden...not enough access rights");
		}
		catch (IOException e) {
			response.setCode(MatpConstants.MATP_SERVER_ERROR);
			response.setComment("I/O errors");
		}
		return response;
	}

	protected Response handleUnknownMethod(String method) {
		ResponseFactory response = new ResponseFactory();
		response.setProtocol("HTTP");
		response.setVersion("1.0");
		response.setCode(MatpConstants.MATP_BAD_REQUEST);
		response.setComment("Unknown method: " + method);
		return response;
	}

	public void setAgentManager(AgentManager manager) {
		this.manager = manager;
	}
}