/*
 * Copyright (c) 2005 Ichiro Satoh
 * All Rights Reserved.
 * written by Ichiro Satoh
 *
 * This program was developed by Ichiro Satoh at National Institute 
 * of Informatics as a part of a mobile object system, titled 
 * "AgentSpace".  All the programs of the system is developed by 
 * Ichiro Satoh and all rights are reserved by Ichiro Satoh. 
 *
 * Permission to use, copy, modify and distribute this software and
 * its documentation is hereby granted, provided that both the
 * copyright notice and this permission notice appear in all copies of
 * the software, derivative works or modified versions, and any
 * portions thereof, and that both notices appear in supporting 
 * documentation.
 */

/**
 * @author Ichiro Satoh
 * Created on 2005/05/7
 */

package agentspace.runtime;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.jar.JarEntry;
import java.util.jar.JarInputStream;
import java.util.jar.JarOutputStream;
import java.util.zip.ZipException;

import agentspace.AgentIdentifier;

public class AgentByteUtil {
	public static final String SERIALIZED_AGENT = "SERIALIZED-AGENT.scr";

	public static final String AGENT_NAME = "META-INF/AGENT-NAME.MF"; 
	public static final String AGENT_CLASS_NAME = "META-INF/AGENT-CLASS-NAME.MF";
	public static final String AGENT_STATUS = "META-INF/AGENT-STATUS.MF";
	public static final String AGENT_ID = "META-INF/AGENT-ID.MF";
	public static final String AGENT_DATE = "META-INF/AGENT-DATE.MF";
	
	public static void main(String[] args) {}
	

	public static byte[] serialize(AgentContainer ac) {
		byte[] data = null;
		try {
			ByteArrayOutputStream baos = new ByteArrayOutputStream();
			JarOutputStream jos = new JarOutputStream(baos);
			if (ac.getCodes() != null) {
				Enumeration enum = ac.getCodes().keys();
				while(enum.hasMoreElements()) {
					String fileName = (String)enum.nextElement();
					String className = fileName.replace('.','/')+".class";
					byte[] classData = (byte[])ac.getCodes().get(fileName);
					jos.putNextEntry(new JarEntry(className));
					jos.write(classData, 0, classData.length);
				}
			}
			if (ac.getResources() != null) {
				Enumeration enum = ac.getResources().keys();
				while(enum.hasMoreElements()) {
					String resourceName = (String)enum.nextElement();
					byte[] resourceData = (byte[])ac.getResources().get(resourceName);
					jos.putNextEntry(new JarEntry(resourceName));
					jos.write(resourceData, 0, resourceData.length);
				}
			}
			if (ac.getState() != null) {
				jos.putNextEntry(new JarEntry(SERIALIZED_AGENT));
				jos.write(ac.getState(), 0, ac.getState().length);
			}
			if (ac.getName() != null) {
				byte[] buf = ac.getName().getBytes();
				jos.putNextEntry(new JarEntry(AGENT_NAME));
				jos.write(buf, 0, buf.length);
			}
			if (ac.getAgentClassName() != null) {
				byte[] buf = ac.getAgentClassName().getBytes();
				jos.putNextEntry(new JarEntry(AGENT_CLASS_NAME));
				jos.write(buf, 0, buf.length);
			}
			if (ac.getStatus() != AgentStatus.UNKNOWN) {
				byte[] buf = AgentStatus.getLifeName(ac.getStatus()).getBytes();
				jos.putNextEntry(new JarEntry(AGENT_STATUS));
				jos.write(buf, 0, buf.length);
			}
			if (ac.getIdentifier() != null) {
				byte[] buf = ac.getIdentifier().toString().getBytes();
				jos.putNextEntry(new JarEntry(AGENT_ID));
				jos.write(buf, 0, buf.length);
			}
			if (ac.getTime() > 0) {
				byte[] buf = ac.getTimeFormat().getBytes();
				jos.putNextEntry(new JarEntry(AGENT_DATE));
				jos.write(buf, 0, buf.length);
			}
			jos.closeEntry();
			jos.close();
			data = baos.toByteArray();
			baos.close();
		}
		catch (ZipException ex) {
			ex.printStackTrace();
		}
		catch (IOException ex) {
			ex.printStackTrace();
		}
		return data;
	}		

	public static AgentContainer deserialize(byte[] data) {
		AgentContainer ac = new AgentContainer();
		try {
			ByteArrayInputStream bais = new ByteArrayInputStream(data);
			JarInputStream jin = new JarInputStream(bais);
			JarEntry entry;
			while((entry = (JarEntry)jin.getNextEntry()) != null) {
				String entryName = entry.getName();
				ByteArrayOutputStream baos = new ByteArrayOutputStream();
				byte[] buf = new byte[1024];
				int size;
				while ((size = jin.read(buf)) != -1) {
					baos.write(buf, 0, size);
				}
				byte[] entryData = baos.toByteArray();
//				System.err.println("AgentPacketUtil: deserialize entryName = "+entryName+", size = "+entryData.length);
				if (entryName.startsWith(AGENT_NAME)) {
					ac.setName(new String(entryData));
				}
				else if (entryName.startsWith(AGENT_CLASS_NAME)) {
					ac.setAgentClassName(new String(entryData));
				}
				else if (entryName.startsWith(AGENT_ID)) {
					ac.setIdentifier(new AgentIdentifier((new String(entryData))));
				}
				else if (entryName.startsWith(AGENT_DATE)) {
					ac.setTimeFormat(new String(entryData));
				}	
				else if (entryName.startsWith(AGENT_STATUS)) {
					ac.setStatus(AgentStatus.getLifeStatus(new String(entryData)));
				}
				else if (entryName.equals(SERIALIZED_AGENT)) {
					ac.setState(entryData);
				}
				else if (entryName.endsWith(".class")) {
					if (ac.getCodes() == null) {
						ac.setCodes(new Hashtable());
					}
					String className = entryName.substring(0, entryName.length() - 6);
					className = className.replace('/', '.');
					ac.getCodes().put(className, entryData);
				}
				else {
					if (ac.getResources() == null) {
						ac.setResources(new Hashtable());
					}
					ac.getResources().put(entryName, entryData);
				}
			}
			jin.close();
			bais.close();
		} 
		catch (IOException e) {
			e.printStackTrace();
		}
//		System.err.println("AgentPacketUtil: deserialize ac = "+ac);
		return ac;
	}
}
