/*
 * Copyright (c) 2005 Ichiro Satoh
 * All Rights Reserved.
 * written by Ichiro Satoh
 *
 * This program was developed by Ichiro Satoh at National Institute 
 * of Informatics as a part of a mobile object system, titled 
 * "AgentSpace".  All the programs of the system is developed by 
 * Ichiro Satoh and all rights are reserved by Ichiro Satoh. 
 *
 * Permission to use, copy, modify and distribute this software and
 * its documentation is hereby granted, provided that both the
 * copyright notice and this permission notice appear in all copies of
 * the software, derivative works or modified versions, and any
 * portions thereof, and that both notices appear in supporting 
 * documentation.
 */

/**
 * @author Ichiro Satoh
 * Created on 2005/05/7
 */

package agentspace.runtime.loader;

import java.util.Hashtable;

public class HashtableClassLoader extends ClassLoader {
    private Hashtable cache;
    private Hashtable codes;

    public HashtableClassLoader(Hashtable codes) {
    	this.codes = codes;
    	cache = new Hashtable();
    }

    public Class findClass(String name) throws ClassNotFoundException {
        return loadClass(name);
    }

    public synchronized Class loadClass(String name) throws ClassNotFoundException {
        return loadClass(name, true);
    }

    /* 
     * @see java.lang.ClassLoader#loadClass(java.lang.String, boolean)
     */
    public synchronized Class loadClass(String name, boolean resolve)
            throws ClassNotFoundException {
    	if (name == null) {
    		throw new ClassNotFoundException("Null class cannot loaded.");
    	}
//    	System.err.println("HashClassLoader: name = "+name);
        Class newClass = (Class)cache.get(name);
        if (newClass == null) {
        	if (codes != null) {
        		byte[] data = (byte[])codes.get(name);
        		if (data != null) { 
        			newClass = defineClass(name, data, 0, data.length);
//        			System.err.println("HashClassLoader: name = "+name+" class "+newClass);
        			cache.put(name, newClass);
        		}
        	}
        }
        if (newClass == null) {
            newClass = findSystemClass(name);
			cache.put(name, newClass);
        }
        if (resolve) {
            resolveClass(newClass);
        }
        return newClass;
    }
}